/*
 * Scratchpad MMU - SMMU simulator
 * Copyright (C) Jack Whitham 2009
 * http://www.jwhitham.org.uk/c/smmu.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation
 * (version 2.1 of the License only).
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
#ifndef SMMU_H
#define SMMU_H

#include "mb_core.h"
#include "mb_mem.h"

#define ACTIVE_BITS     24
#define ACTIVE_MASK     (~((1 << ACTIVE_BITS) - 1))
#define CONTROL_AREA    0xa0000000UL
#define SCRATCHPAD_AREA 0xc0000000UL
#define ACTIVE_AREA     0x90000000UL


#define COMMENT_SIZE    128

struct SMMU_Context_struct ;

struct SMMU_Context_struct * SMMU_Init ( 
        struct MB_System_Context_struct * sys ,
        unsigned num_rw_entries ,
        unsigned num_ro_entries ,
        unsigned w_sp_size ) ;
void SMMU_Delete ( struct SMMU_Context_struct * sc ) ;
void SMMU_Cache_Init (struct SMMU_Context_struct * sc,
        unsigned log2_cache_lines_per_page,
        unsigned log2_cache_words_per_line);
unsigned SMMU_Open (
        struct SMMU_Context_struct * sc ,
        unsigned notional_base_pointer ,
        unsigned w_size ,
        unsigned w_copy_location ,
        int offset_hint ,
        unsigned comment_ptr ,
        int read_only ) ;
void SMMU_Close (
        struct SMMU_Context_struct * sc ,
        unsigned ref ) ;
int SMMU_Inspect_Table ( struct SMMU_Context_struct * sc ,
        unsigned ref ,
        unsigned * min_address ,
        unsigned * max_address ,
        unsigned * base ,
        unsigned * opened_at ,
        char * comment_copy ) ;
void SMMU_Dump_Table ( struct SMMU_Context_struct * sc ) ;

#endif

